//=============================================================================
// Unreal Texture Upgrade Mutator
// Gerald Lance Tindall aka. {KDS}BOZO aka. <NDP>BOZO aka. LordBozo
// bozo@kds-clan.com
// CODE NOT FOR REUSE WITHOUT PRIOR PERMISSION OF THE AUTHOR
//=============================================================================
// UnrealTextureUpgrade.UTUClient:
// Allows the dynamic changeout of the textures that will be utilized
// for any mesh referenced by the supported classnames at runtime.
// The replacement funcionality is client specific to allow for optional S3TC use.
// Designed to support regular and/or S3TC textures.
//
// VERSION 0.1 BETA
//
//=============================================================================
class UTUClient expands Actor config (UnrealTextureUpgrade);

// NOTE: Add a check vs correct Mesh within class also.

struct TexReplacer_type
{
   var() config string TClass;   // Target Class

   var() config string R0_Name;  // Replacement Texture Name (Slot 0)
   var() config string R1_Name;  // Replacement Texture Name (Slot 1)
   var() config string R2_Name;  // Replacement Texture Name (Slot 2)
   var() config string R3_Name;  // Replacement Texture Name (Slot 3)
   var() config string R4_Name;  // Replacement Texture Name (Slot 4)
   var() config string R5_Name;  // Replacement Texture Name (Slot 5)
   var() config string R6_Name;  // Replacement Texture Name (Slot 6)
   var() config string R7_Name;  // Replacement Texture Name (Slot 7)

   var() config string D0_Name;  // Default Texture Name (Slot 0)
   var() config string D1_Name;  // Default Texture Name (Slot 1)
   var() config string D2_Name;  // Default Texture Name (Slot 2)
   var() config string D3_Name;  // Default Texture Name (Slot 3)
   var() config string D4_Name;  // Default Texture Name (Slot 4)
   var() config string D5_Name;  // Default Texture Name (Slot 5)
   var() config string D6_Name;  // Default Texture Name (Slot 6)
   var() config string D7_Name;  // Default Texture Name (Slot 7)

};

var() config TexReplacer_type TexReplacer[64];

var int NumReplacementClasses;
var int CurrentReplacementClass;

var UTUTexRef UTUTexRefList;
var UTUTexRef UTUR;

var int NumReplacementActors;

var UTUActorListElement UTUActorList;
var UTUActorListElement UTUA;

var() config float checktime;
var() config float viewradius;
var() config bool Debug;

//=============================================================================
// UTUClient.PreBeginPlay
//=============================================================================
event PreBeginPlay()
{
   local int i, j;

   if( Level.Netmode != NM_DedicatedServer && Level.Netmode != NM_ListenServer )
   {
      if(PlayerPawn(Owner)==None)
      {
         self.Destroy();
      }// endif

      NumReplacementClasses=0;

      for (i=0;i<64;i++)
      {
         if(TexReplacer[i].TClass!="")
         {
            if(UTUTexRefList!=None)
            {
               UTUR.Next=new(self) class 'UTUTexRef';
               UTUR=UTUR.Next;
               UTUR.Next=None;
               UTUR.TClass=TexReplacer[i].TClass;
            }else
            {
               UTUTexRefList=new(self) class 'UTUTexRef';
               UTUTexRefList.Next=None;
               UTUR=UTUTexRefList;
               UTUR.TClass=TexReplacer[i].TClass;
            }// endif

            NumReplacementClasses++;
            for (j=0;j<8;j++)
            {
               if(j==0)
               {
                  if(TexReplacer[i].D0_Name!="")
                  {
                     UTUR.D0_Name=TexReplacer[i].D0_Name;
                  }else
                  {
                     UTUR.D0_Name="";
                  }// endif
                  if(TexReplacer[i].R0_Name!="")
                  {
                     UTUR.R0_Name=TexReplacer[i].R0_Name;
                  }else
                  {
                     UTUR.R0_Name="";
                  }// endif
               }else if(j==1)
               {
                  if(TexReplacer[i].D1_Name!="")
                  {
                     UTUR.D1_Name=TexReplacer[i].D1_Name;
                  }else
                  {
                     UTUR.D1_Name="";
                  }// endif
                  if(TexReplacer[i].R1_Name!="")
                  {
                     UTUR.R1_Name=TexReplacer[i].R1_Name;
                  }else
                  {
                     UTUR.R1_Name="";
                  }// endif
               }else if(j==2)
               {
                  if(TexReplacer[i].D2_Name!="")
                  {
                     UTUR.D2_Name=TexReplacer[i].D2_Name;
                  }else
                  {
                     UTUR.D2_Name="";
                  }// endif
                  if(TexReplacer[i].R2_Name!="")
                  {
                     UTUR.R2_Name=TexReplacer[i].R2_Name;
                  }else
                  {
                     UTUR.R2_Name="";
                  }// endif
               }else if(j==3)
               {
                  if(TexReplacer[i].D3_Name!="")
                  {
                     UTUR.D3_Name=TexReplacer[i].D3_Name;
                  }else
                  {
                     UTUR.D3_Name="";
                  }// endif
                  if(TexReplacer[i].R3_Name!="")
                  {
                     UTUR.R3_Name=TexReplacer[i].R3_Name;
                  }else
                  {
                     UTUR.R3_Name="";
                  }// endif
               }else if(j==4)
               {
                  if(TexReplacer[i].D4_Name!="")
                  {
                     UTUR.D4_Name=TexReplacer[i].D4_Name;
                  }else
                  {
                     UTUR.D4_Name="";
                  }// endif
                  if(TexReplacer[i].R4_Name!="")
                  {
                     UTUR.R4_Name=TexReplacer[i].R4_Name;
                  }else
                  {
                     UTUR.R4_Name="";
                  }// endif
               }else if(j==5)
               {
                  if(TexReplacer[i].D5_Name!="")
                  {
                     UTUR.D5_Name=TexReplacer[i].D5_Name;
                  }else
                  {
                     UTUR.D5_Name="";
                  }// endif
                  if(TexReplacer[i].R5_Name!="")
                  {
                     UTUR.R5_Name=TexReplacer[i].R5_Name;
                  }else
                  {
                     UTUR.R5_Name="";
                  }// endif
               }else if(j==6)
               {
                  if(TexReplacer[i].D6_Name!="")
                  {
                     UTUR.D6_Name=TexReplacer[i].D6_Name;
                  }else
                  {
                     UTUR.D6_Name="";
                  }// endif
                  if(TexReplacer[i].R6_Name!="")
                  {
                     UTUR.R6_Name=TexReplacer[i].R6_Name;
                  }else
                  {
                     UTUR.R6_Name="";
                  }// endif 
               }else if(j==7)
               {
                  if(TexReplacer[i].D7_Name!="")
                  {
                     UTUR.D7_Name=TexReplacer[i].D7_Name;
                  }else
                  {
                     UTUR.D7_Name="";
                  }// endif
                  if(TexReplacer[i].R7_Name!="")
                  {
                     UTUR.R7_Name=TexReplacer[i].R7_Name;
                  }else
                  {
                     UTUR.R7_Name="";
                  }// endif
               }// endif
            }// end for

            if(Debug)
            {
               log(i@UTUR.TClass@"LOADING DATA");
               log("MultiSkins[0] DEFAULT="@UTUR.D0_Name);
               log("MultiSkins[1] DEFAULT="@UTUR.D1_Name);
               log("MultiSkins[2] DEFAULT="@UTUR.D2_Name);
               log("MultiSkins[3] DEFAULT="@UTUR.D3_Name);
               log("MultiSkins[4] DEFAULT="@UTUR.D4_Name);
               log("MultiSkins[5] DEFAULT="@UTUR.D5_Name);
               log("MultiSkins[6] DEFAULT="@UTUR.D6_Name);
               log("MultiSkins[7] DEFAULT="@UTUR.D7_Name);
               log("MultiSkins[0] REPLACEMENT="@UTUR.R0_Name);
               log("MultiSkins[1] REPLACEMENT="@UTUR.R1_Name);
               log("MultiSkins[2] REPLACEMENT="@UTUR.R2_Name);
               log("MultiSkins[3] REPLACEMENT="@UTUR.R3_Name);
               log("MultiSkins[4] REPLACEMENT="@UTUR.R4_Name);
               log("MultiSkins[5] REPLACEMENT="@UTUR.R5_Name);
               log("MultiSkins[6] REPLACEMENT="@UTUR.R6_Name);
               log("MultiSkins[7] REPLACEMENT="@UTUR.R7_Name);
               log("--------------------------------------------------------------------------------------");
            }// endif
         }else
         {
            if(Debug)
            {
               log(i@TexReplacer[i].TClass@"NO DATA");
               log("--------------------------------------------------------------------------------------");
            }// endif
         }// endif
      }// end for

      if(UTUTexRefList!=None)
      {
         if(Debug)
         {
            UTUR=UTUTexRefList;
            log("UTUClient: Classes that have Replacement Textures Designated :");
            for(i=0;i<NumReplacementClasses;i++)
            {
               log("UTUClient:"@i@")"@UTUR.TClass);
               UTUR=UTUR.Next;
            }// end for
            log("--------------------------------------------------------------------------------------");
         }// endif
         GoToState('CheckforNewMeshes');

      }else
      {
         log("UTUClient: No Classes have been listed on Client in UTUMutator.ini as having Available Replacement Textures. Terminating UTUClient."); 
         self.Destroy();
      }// endif
   }// endif

}// end event UTUClient.PreBeginPlay

//=============================================================================
// UTUClient:CheckforNewMeshes
//=============================================================================
auto state CheckforNewMeshes
{

//=============================================================================
// UTUClient:CheckforNewMeshes.BeginState()
//=============================================================================
function BeginState ()
{
   if(Debug)
   {
      log("-- UTUClient:CheckforNewMeshes.BeginState() >> "$Owner$" --");
      log("Role :"@Role@"Netmode :"@Level.Netmode);
      log("======================================================================================");
      log("======================================================================================");
   }// endif

   CurrentReplacementClass=0;
   UTUR=UTUTexRefList;

   UTUActorList=None;
   UTUA=None;

   NumReplacementActors=0;

   viewradius=fclamp(viewradius,200.0,1000.0);

   SetTimer(checktime/NumReplacementClasses,True);
}// end function UTUClient:CheckforNewMeshes.BeginState


//=============================================================================
// UTUClient:CheckforNewMeshes.Timer()
//=============================================================================
function Timer ()
{
   local Actor ActorPtr;
   local UTUTexReplacer TexRepPtr;
   local byte bSuperRelevant;
   local bool bModifiedActor, bReplaceableActor;

   bSuperRelevant=0;

   if( Level.Netmode!=NM_DedicatedServer && Level.Netmode!=NM_ListenServer )
   {
      if(Debug)
      {
         log("-- UTUClient:CheckforNewMeshes.Timer() >> "$Owner$" --");
         log(Level.TimeSeconds);
         log("======================================================================================");
         log("======================================================================================");
      }// endif

      ActorPtr=None;
      TexRepPtr=None;

      foreach Owner.VisibleCollidingActors(Class'Actor',ActorPtr,viewradius)
      {
         if(ActorPtr==None || ActorPtr.IsA('Decoration'))
         {
            // SKIP DECORATIONS
         
         }else if(ActorPtr.IsA('PlayerPawn')||ActorPtr.IsA('Bots'))
         {
            if(Debug)
            {
               log("--------------------------------------------------------------------------------------");
               log("UTUClient: Target Actor:"@ActorPtr@"| Target Class:"@ActorPtr.Class);
               log("UTUClient: PlayerPawn or Bots Textures not allowed to be dynamically replaced via this client instance.");
            }// endif
         }else if(UTUR.TClass==string(ActorPtr.class))
         {
            if(Debug)
            {
               log("--------------------------------------------------------------------------------------");
               log("UTUClient: Target Actor:"@ActorPtr@"| Target Class:"@ActorPtr.Class);
            }//endif

            bModifiedActor=False;
            bReplaceableActor=True;

            if(UTUActorList!=None)
            {
               bModifiedActor=UTUActorList.IsModifiedActor(string(ActorPtr));
            }// endif

            if(!bModifiedActor)
            {
//               bReplaceableActor=CheckReplacement(ActorPtr,bSuperRelevant);

               TexRepPtr=Spawn(Class'UTUTexReplacer', ActorPtr,,ActorPtr.Location+vect(0,0,200));
               TexRepPtr.UTUR=UTUR;

               if(bReplaceableActor)
               {
                  UTUA=new(self) class'UTUActorListElement';
                  UTUA.Next=None;
                  UTUA.ModifiedActor=string(ActorPtr);
                  NumReplacementActors++;

                  if(Debug){ log("UTUClient: ActorCount:"@NumReplacementActors@")"@ActorPtr);}// endif
	
                  if(UTUActorList!=None)
                  {
                     UTUActorList.AddActor(UTUA);
                  }else
                  {
                     UTUActorList=UTUA;
                  }// endif
               }// endif
            }else
            {
               if(Debug){ log("UTUClient: ALREADY MODIFIED");}// endif
            }// endif
         }// endif
      }// end foreach
      

      if(Debug){ log("UTUClient: CurrentReplacementClass ="@CurrentReplacementClass@"| NumReplacementClasses ="@NumReplacementClasses);}// endif
      CurrentReplacementClass++;
      if(CurrentReplacementClass>=NumReplacementClasses)
      {
         CurrentReplacementClass=0;
         UTUR=UTUTexRefList;
      }else
      {
         UTUR=UTUR.Next;
      }// endif
   }// endif

}// end function UTUClient:CheckforNewMeshes.Timer


//=============================================================================
// UTUClient:CheckforNewMeshes.EndState()
//=============================================================================
function EndState ()
{
   if(Debug)
   {
      log("-- UTUClient:CheckforNewMeshes.EndState() >> "$Owner$" --");
      log("======================================================================================");
      log("======================================================================================");
   }// endif

   GoToState('CheckforNewMeshes');
}// end function UTUClient:CheckforNewMeshes.EndState


}// end state UTUClient:CheckforNewMeshes

//-----------------------------------------------------------------------------

//=============================================================================
// UTUClient.CheckReplacement
//=============================================================================
function bool CheckReplacement (Actor Other, out byte bSuperRelevant)
{
   local string S,T,R,D;
   local int i, j;
   local UTUTexReplacer TexRep;

   bSuperRelevant = 0;

   S=string(Other.Class);

   UTUR=FindMatchingClass(S);

   if(UTUR==None)
   {
      if(Debug){ log("UTUClient: Target Actor:"@Other@"| Unsupported Class:"@S);}// endif
      return False;
   }else
   {
      T=UTUR.TClass;

      if(T!=S)
      {
         if(Debug){log("ERROR");}// endif
         return False;   
      }else
      {
         if(Debug){ log("UTUClient: Target Actor:"@Other@"| Supported Replacement Class:"@T);}// endif

         TexRep=Spawn(Class'UTUTexReplacer', Other,,Other.Location+vect(0,0,200));
         TexRep.UTUR=UTUR;

         return True;
      }// endif
   }// endif 
   return False;

}// end function UTUClient.CheckReplacement


//=============================================================================
// UTUClient.FindMatchingClass
//=============================================================================
function UTUTexRef FindMatchingClass(string s)
{
   local UTUTexRef Result;

   UTUR=UTUTexRefList;

   do
   {
      if(s==UTUR.TClass)
      {
         Result=UTUR;
         return (Result);
      }else
      {
         if(UTUR.Next!=None)
         {
            UTUR=UTUR.Next;
         }else
         {
            return None;
         }// endif
      }// endif

   }until(Result!=None)

}// end function UTUClient.FindMatchingClass


//=============================================================================
// UTUClient:defaultproperties
//=============================================================================

defaultproperties
{
				checktime=5.000000
				viewradius=500.000000
				RemoteRole=ROLE_None
				DrawType=DT_None
				bGameRelevant=True
}
